﻿namespace InputValidationTesting.ViewModel {
    using System;
    using System.Collections;
    using System.ComponentModel;
    using System.Runtime.CompilerServices;
    using System.Threading;
    using System.Threading.Tasks;

    using InputValidationTesting.Helper;

    public class CustomerEditViewModel : INotifyPropertyChanged, INotifyDataErrorInfo {

        private readonly ErrorValidationHelper<CustomerEditViewModel> _errorValidationHelper;
        private readonly string _firstnameTemp;
        private readonly string _lastnameTemp;

        private string _firstname;
        private string _lastname;
       
        public CustomerEditViewModel() {
            _firstnameTemp = "Vorname1";
            _lastnameTemp = "Vorname2";
            _errorValidationHelper = new ErrorValidationHelper<CustomerEditViewModel>(this);
        }

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        ///     Liefert oder setzt den Vornamen des Kunden.
        /// </summary>
        [Required(AllowEmptyStrings = false, ErrorMessage = "Vorname wird benötigt!")]
        [MaxLength(256, ErrorMessage = "Maximal 256 Zeichen!")]
        public string Firstname {
            get { return _firstname; }
            set {
                _firstname = value;
                OnPropertyChanged();
            }
        }

        public bool HasErrors {
            get { return _errorValidationHelper.HasErrors; }
        }

        /// <summary>
        ///     Liefert oder setzt den Nachnamen des Kunden.
        /// </summary>
        [Required(AllowEmptyStrings = false, ErrorMessage = "Nachname wird benötigt!")]
        [MaxLength(256, ErrorMessage = "Maximal 256 Zeichen!")]
        public string Lastname {
            get { return _lastname; }
            set {
                _lastname = value;
                OnPropertyChanged();
            }
        }

        public IEnumerable GetErrors(string propertyName) {
            return _errorValidationHelper.GetErrors(propertyName);
        }

        public void InvokeErrorsChanged(DataErrorsChangedEventArgs e) {
            ErrorsChanged?.Invoke(this, e);
        }

        public async void LoadData() {
            await Task.Run(() => {
                               Thread.Sleep(0);
                           });

            Firstname = _firstnameTemp;
            Lastname = _lastnameTemp;
        }

        protected void OnPropertyChanged([CallerMemberName] string propertyName = null) {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

    }
}